//---- Left listbox with mission units -------------------------------------------------

// Display list of sides
ME3D_REFRESH_SIDE_LIST = {
	lbClear 1;
	lbSetCurSel [1, -1];
	
	_sides = [["WEST","STR_WEST"], ["EAST","STR_EAST"], ["GUER","STR_GUERRILA"], ["CIV","STR_CIVILIAN"], ["LOGIC","STR_LOGIC"]];
	"lbSetData [1, lbAdd [1, localize (_x select 1)], _x select 0]" forEach _sides;

	lbSetData [1, lbAdd [1, localize "STR_SENSORACTIV_VEHICLE"], "EMPTY"];
	lbSetData [1, lbAdd [1, loadFile Format ["\:STRING CUT  end:-5  text:%1", localize "STR_DISP_ARCMAP_MARKERS"]], "MARKER"];
	lbSetData [1, lbAdd [1, loadFile Format ["\:STRING CUT  end:-5  text:%1", localize "STR_DISP_ARCMAP_SENSORS"]], "TRIGGER"];

	if (_side_selection >= lbSize 1) then {
		_side_selection = (lbSize 1) - 1
	};

	lbSetCurSel [1, _side_selection];
	call ME3D_REFRESH_GROUP_LIST
};

// Display list groups belonging to the currently selected side
ME3D_REFRESH_GROUP_LIST = {
	lbClear 2;
	lbSetCurSel [2, -1];
	
	private ["_i", "_side"];
	_side = lbData [1, _side_selection];
	
	if (_side in ["WEST", "EAST", "GUER", "CIV", "LOGIC"]) then {
		_i = -1;
		while "_i=_i+1; _i < count ME3D_GROUPS_SIDE" do {
			_group_side = ME3D_GROUPS_SIDE select _i;
			if (_group_side == _side) then {
				_index      = lbAdd [2, Format ["Group #%1 (%2 %3)", _i, count (ME3D_GROUPS_OBJ select _i), localize "STR_DISP_SRVSETUP_UNITS"]];
				
				// saving index so later we can access this group from the array of all groups (ME3D_GROUPS_OBJ)
				lbSetValue [2, _index, _i];	
				
				// saving group number so later we can access this group in the sqm file
				lbSetData  [2, _index, Format ["%1", _i]]
			};
		};
	};
	
	if (_side == "EMPTY") then {
		lbAdd [2, Format ["%1 %2", count ME3D_VEHICLES_OBJ, localize "STR_SENSORACTIV_VEHICLE"]];
	};
	
	if (_side == "MARKER") then {
		lbAdd [2, Format ["%1 %2", count ME3D_MARKERS_NAME, loadFile Format ["\:STRING CUT  end:-5  text:%1", localize "STR_DISP_ARCMAP_MARKERS"]]];
	};
	
	if (_side == "TRIGGER") then {
		lbAdd [2, Format ["%1 %2", count ME3D_SENSORS_TEXT, loadFile Format ["\:STRING CUT  end:-5  text:%1", localize "STR_DISP_ARCMAP_SENSORS"]]];
	};		
	
	if (_group_selection >= lbSize 2) then {
		_group_selection = (lbSize 2) - 1
	};

	lbSetCurSel [2, _group_selection];
	true call ME3D_REFRESH_UNIT_LIST
};

// Display list of objects/waypoints belonging to the currently selected group
ME3D_REFRESH_UNIT_LIST = {
	"lbClear _x; lbSetCurSel [_x, -1]" forEach [3,4];

	if (lbSize 2 > 0) then {
		private ["_i", "_soldiers", "_waypoints", "_group_index", "_unit_list", "_waypoint_list"];
		_i    = -1;
		_side = lbData [1, _side_selection];

		if (_side in ["WEST", "EAST", "GUER", "CIV", "LOGIC"]) then {
			_group_index = lbValue [2, lbCurSel 2];
			_unit_list   = ME3D_GROUPS_OBJ select _group_index;
			"lbAdd [3, Format [""%1"",_x]]" forEach _unit_list;
			
			_waypoint_list = ME3D_GROUPS_WP select _group_index;
			"lbAdd [4, Format [""%1"",_x select 1]]" forEach _waypoint_list;
		};
		
		if (_side == "EMPTY") then {
			"lbAdd [3, _x]" forEach ME3D_VEHICLES_NAME
		};
		
		if (_side == "MARKER") then {
			"lbAdd [3, _x]" forEach ME3D_MARKERS_NAME
		};
		
		if (_side == "TRIGGER") then {
			{lbAdd [3, (if (_x=="") then {localize "STR_SENSOR"} else {_x})]} forEach ME3D_SENSORS_TEXT
		};	
		
		//player groupChat Format ["_unit_selection: %1 >= %2", _unit_selection, lbSize 3];
			
		if (_unit_selection >= lbSize 3) then {
			_unit_selection = (lbSize 3) - 1;
			//player groupChat Format ["_unit_selection: %1 after", _unit_selection];
		};

	//player groupChat Format ["_waypoint_selection: %1 >= %2", _waypoint_selection, lbSize 4];
		if (_waypoint_selection >= lbSize 4) then {
			_waypoint_selection = (lbSize 4) - 1;
			//player groupChat Format ["_waypoint_selection: %1 after", _waypoint_selection];
		};
		
		if (ME3D_CURRENT_SELECTION_TYPE == "soldier") then {
			lbSetCurSel [3, _unit_selection];
		} else {
			lbSetCurSel [4, _waypoint_selection];
		};
		
		//player globalChat Format ["ME3D_CURRENT_SELECTION_TYPE:%1", ME3D_CURRENT_SELECTION_TYPE];
		
		// argument passed to this function determines if camera is going to be refreshed
		if (_this) then {
			ME3D_CURRENT_SELECTION_TYPE call ME3D_REFRESH_UNIT_SELECTION
		}
	}
};

// Clear unit listbox and fill it with units belonging to the current group
ME3D_RESET_UNIT_LIST = {
	private ["_side","_group_index", "_unit_list"];
	_side = lbData [1, _side_selection];
	"lbClear _x; lbSetCurSel [_x, -1]" forEach [3];
	if (_side in ["WEST", "EAST", "GUER", "CIV", "LOGIC"]) then {
		_group_index = lbValue [2, lbCurSel 2];
		_unit_list   = ME3D_GROUPS_OBJ select _group_index;
		"lbAdd [3, Format [""%1"",_x]]" forEach _unit_list;
	};
	
	if (_side == "EMPTY") then {
		"lbAdd [3, _x]" forEach ME3D_VEHICLES_NAME
	};
	
	if (_side == "MARKER") then {
		"lbAdd [3, _x]" forEach ME3D_MARKERS_NAME
	};
	
	if (_side == "TRIGGER") then {
		"lbAdd [3, _x]" forEach ME3D_SENSORS_TEXT
	};	
};

// Clear waypoint listbox and fill it with waypoints belonging to the current group
ME3D_RESET_WAYPOINT_LIST = {
	private ["_side","_waypoint_list"];
	_side = lbData [1, _side_selection];
	"lbClear _x; lbSetCurSel [_x, -1]" forEach [4];
	if (_side in ["WEST", "EAST", "GUER", "CIV", "LOGIC"]) then {
		_waypoint_list = ME3D_GROUPS_WP select _group_index;
		"lbAdd [4, Format [""%1"",_x select 1]]" forEach _waypoint_list;
	};

};

// Change camera target
ME3D_REFRESH_UNIT_SELECTION = {
	private ["_side", "_group_index", "_ok", "_output_value0", "_angle", "_i"];
	_side                       = lbData  [1, _side_selection];
	_group_index                = lbValue [2, _group_selection];
	ME3D_CURRENT_SELECTION_TYPE = _this;

	if (_this == "soldier") then {
		_waypoint_selection = -1;
		//false call ME3D_REFRESH_UNIT_LIST;
		call ME3D_RESET_WAYPOINT_LIST;
		
		if (_unit_selection >= 0) then {
			if (_side in ["WEST", "EAST", "GUER", "CIV", "LOGIC"]) then {
				ME3D_CAMERA_TARGET = (ME3D_GROUPS_OBJ select _group_index) select _unit_selection;
				[_side, _group_index, _unit_selection, (getPos ME3D_CAMERA_TARGET select 0), (getPos ME3D_CAMERA_TARGET select 1), 0, true] call ME3D_SETPOS;
			};
		
			if (_side == "EMPTY") then {
				ME3D_CAMERA_TARGET = ME3D_VEHICLES_OBJ select _unit_selection;
				[_side, _group_index, _unit_selection, (getPos ME3D_CAMERA_TARGET select 0), (getPos ME3D_CAMERA_TARGET select 1), 0, true] call ME3D_SETPOS;
			};
			
			if (_side == "MARKER") then {
				_output_property0 = [];
				_output_value0    = [];
				_ok               = call loadFile Format ["\:CLASS READ  file:mission.sqm  path:[Mission,Markers,Item%1]  find:position[],angle,type  wrap:no", _unit_selection];
				
				if (_ok select 0) then {
					_i = -1;
					_object = (if (ME3D_ADDON_PRESENT) then {ME3D_GROUND_TEX} else {ME3D_PROXY_TARGET});
					_object setDir 0;
					
					while "_i=_i+1; _i<count _output_property0" do {
						if ((_output_property0 select _i) == "position[]") then {
							//ME3D_PROXY_TARGET setPos [(_output_value0 select _i) select 0, (_output_value0 select _i) select 2, 0];
							_object setPos [(_output_value0 select _i) select 0, (_output_value0 select _i) select 2, 0];
						};
						
						if ((_output_property0 select _i) == "angle") then {
							//ME3D_PROXY_TARGET setDir (_output_value0 select _i);
							_object setDir (_output_value0 select _i);
						};
						
						if (ME3D_ADDON_PRESENT && (_output_property0 select _i) == "type") then {
							_index = [(_output_value0 select _i), ME3D_EDITMARKER_TYPE] call ME3D_FIND;
							
							if (_index >= 0) then {
								ME3D_GROUND_TEX setObjectTexture [1, ME3D_EDITMARKER_TYPE_TEX select _index];
							} else {
								ME3D_GROUND_TEX setObjectTexture [1, ""];
							};
						}
					};
									
					//ME3D_CAMERA_TARGET = ME3D_PROXY_TARGET;
					ME3D_CAMERA_TARGET = _object;
				}
			};
			
			if (_side == "TRIGGER") then {
				_output_value0    = [];
				_ok               = call loadFile Format ["\:CLASS READ  file:mission.sqm  path:[Mission,Sensors,Item%1]  find:position[]  wrap:no", _unit_selection];
				_object           = (if (ME3D_ADDON_PRESENT) then {ME3D_GROUND_TEX setObjectTexture [1, "vlajka.paa"]; ME3D_GROUND_TEX} else {ME3D_PROXY_TARGET});

				if (count _output_value0 > 0) then {
					_object setDir 0;
					_object setPos [(_output_value0 select 0) select 0, (_output_value0 select 0) select 2, 0]
				} else {
					player groupchat format ["%1", _ok]
				};
				
				ME3D_CAMERA_TARGET = _object;
			};			
		};			
	} else {
		_unit_selection = -1;
		//false call ME3D_REFRESH_UNIT_LIST;
		call ME3D_RESET_UNIT_LIST;
		
		if (_waypoint_selection>=0  &&  _waypoint_selection<count (ME3D_GROUPS_WP select _group_index)) then {
			_array = ((ME3D_GROUPS_WP select _group_index) select _waypoint_selection);
			ME3D_PROXY_TARGET setPos (_array select 0);
			ME3D_CAMERA_TARGET = ME3D_PROXY_TARGET;
		};
	};
	
	ME3D_CAMERA_TARGET_ORIGINAL_POS = getPos ME3D_CAMERA_TARGET;
	ME3D_CAMERA_TARGET_ORIGINAL_DIR = getDir ME3D_CAMERA_TARGET;
	
	if (_unit_selection >= 0) then {
		if (_side in ["WEST", "EAST", "GUER", "CIV", "LOGIC"]) then {				
			ME3D_CAMERA_TARGET_ORIGINAL_POS set [2, (ME3D_GROUPS_Z select _group_index) select _unit_selection];
		};

		if (_side == "EMPTY") then {		
			ME3D_CAMERA_TARGET_ORIGINAL_POS set [2, ME3D_VEHICLES_Z select _unit_selection];
		};	
	}
};








//---- Right listbox with general object list ------------------------------------------

// Display list of categories for the current addon
ME3D_REFRESH_CATEGORY_LIST = {
	lbClear 7;
	lbSetCurSel [7, -1];
	
	if (_addon_selection >= 0) then {
		{lbAdd [7, ME3D_LIST_VEHICLECLASS select _x]} forEach (ME3D_BROWSER_PBO_VEHICLECLASS select _addon_selection);
	};
	
	if (lbSize 7 == 0) then {
		_category_selection = -1;
	} else {
		_category_selection = lbValue [6, _addon_selection];
		lbSetCurSel [7, _category_selection];
	};
	
	_this call ME3D_REFRESH_BROWSEUNIT_LIST;
};

// Display list of units for the current category
ME3D_REFRESH_BROWSEUNIT_LIST = {
	private ["_index", "_id", "_current_category"];
	lbClear 8;
	lbSetCurSel [8, -1];

	if (_category_selection >= 0) then {
		lbSetValue [6, _addon_selection, _category_selection];

		_current_addon    = (ME3D_BROWSER_PBO select _addon_selection);
		_current_category = (ME3D_BROWSER_PBO_VEHICLECLASS select _addon_selection) select _category_selection;
		
		{
			_id = lbAdd [8, ME3D_OBJLIST_NAME select _x];
			lbSetData   [8, _id, (ME3D_OBJLIST_CLASS select _x)];
			lbSetValue  [8, _id, _x];
		} 
		forEach ((ME3D_BROWSER_PBO_VEHICLECLASS_DISPLAYNAME select _addon_selection) select _category_selection)
	};
	
	if ("dontresetobjectselection" in _this) then {
		if (_browseunit_selection >=0) then {
			lbSetCurSel [8, _browseunit_selection];
		}
	} else {
		_browseunit_selection = -1;
	};
};

// Delete current preview object and replace it with a new one
ME3D_REFRESHME3D_TEMPOBJ = {
	private ["_class", "_index", "_dont_update_position", "_pos", "_dir", "_side", "_group_id", "_unit_id"];

	_class                = _this select 0;
	_index                = _this select 1;
	_dont_update_position = _this select 2;	
	_pos                  = getPos ME3D_CAMERA_TARGET;
	_dir                  = getDir ME3D_CAMERA_TARGET;

	// If replacing one preview object with another
	if (!isNull ME3D_TEMPOBJ) then {
		if (ME3D_TEMPOBJ != ME3D_PROXY_TARGET) then {
			deleteVehicle ME3D_TEMPOBJ
		}
	} else {
		// if creating a new preview object then take position of the currently select object
		if (!_dont_update_position) then {
			ME3D_TEMPOBJ_DIR    = _dir;
			ME3D_TEMPOBJ_POS    = _pos;
			ME3D_TEMPOBJ_Z_TYPE = 1;
			_side               = lbData [1, _side_selection];
			_group_id           = lbValue [2, lbCurSel 2];
			_unit_id            = lbCurSel 3;
			
			if (ME3D_CURRENT_SELECTION_TYPE == "soldier"  &&  _unit_id>=0) then {
				if (_side in ["WEST", "EAST", "GUER", "CIV", "LOGIC"]) then {
					ME3D_TEMPOBJ_POS set [2, (ME3D_GROUPS_Z select _group_id) select _unit_id];
					ME3D_TEMPOBJ_Z_TYPE = (ME3D_GROUPS_Z_TYPE select _group_id) select _unit_id;
				};
				
				if (_side == "EMPTY") then {
					ME3D_TEMPOBJ_POS set [2, ME3D_VEHICLES_Z select _unit_id];
					ME3D_TEMPOBJ_Z_TYPE = ME3D_VEHICLES_Z_TYPE select _unit_id;
				}
			}
		}
	};
	
	// Create new object. If it's a parachute then use proxy object instead
	ME3D_TEMPOBJ = if ((ME3D_LIST_SIMULATIONS select (ME3D_OBJLIST_SIMULATION select _index)) == "parachute") then {ME3D_PROXY_TARGET} else {_class camCreate [0,0,0]};
	
	if (!isNull ME3D_TEMPOBJ) then {
		ME3D_CAMERA_TARGET = ME3D_TEMPOBJ;
		
		// If it's a soldier/vehicle then it shouldn't be underground
		if ((ME3D_LIST_SIMULATIONS select (ME3D_OBJLIST_SIMULATION select _index)) in ["airplane", "car", "helicopter", "motorcycle", "ship", "soldier", "tank"]) then {
			if (ME3D_TEMPOBJ_Z_TYPE == 1) then {
				if (ME3D_TEMPOBJ_POS select 2 < 0) then {ME3D_TEMPOBJ_POS set [2, 0]}
			};
			
			if (ME3D_TEMPOBJ_Z_TYPE == 2) then {
				ME3D_CAMERA_TARGET setPosASL ME3D_TEMPOBJ_POS;

				if ((getPos ME3D_CAMERA_TARGET select 2) < 0) then {
					ME3D_TEMPOBJ_POS set [2, (ME3D_TEMPOBJ_POS select 2)+abs (getPos ME3D_CAMERA_TARGET select 2)]
				};
			};
		};
		
		[0, 0, 0, ME3D_TEMPOBJ_POS select 0, ME3D_TEMPOBJ_POS select 1, ME3D_TEMPOBJ_POS select 2, false] call ME3D_SETPOS;
	}
};








//---- Sub-menus (edit unit/waypoint/trigger/marker/effects) ---------------------------

// Read mission settings from the mission.sqm and display them in a dialog
ME3D_EDIT_INTEL_REFRESH = {
	private ["_i", "_ok", "_output_property0", "_output_value0", "_value", "_idc"];
	_i                 = -1;
	_output_property0  = [];
	_output_value0     = [];
	_ok                = call loadFile Format ["\:CLASS READ  file:mission.sqm  path:[Mission,Intel]  wrap:no"];
	ME3D_INTEL_CURRENT = [];

	while "_i=_i+1; _i<count ME3D_INTEL_IDC" do {
		_idc          = ME3D_INTEL_IDC  select _i;
		_name_to_find = ME3D_INTEL_VARS select _i;
		_index        = [_name_to_find, _output_property0] call ME3D_FIND;
		_value        = if (_index >= 0) then {_output_value0 select _index} else {ME3D_INTEL_DEFAULTS select _i};
		
		// Save original value for later comparison when saving
		ME3D_INTEL_CURRENT set [_i, _value];
		
		// Update lists
		if (_idc in [101,102,103]) then {
			if (_idc in [102,103]) then {_value=_value-1};
			
			// refresh days in month
			if (_idc == 102) then {
				_value call ME3D_EDIT_INTEL_UPDATE_DAYS_IN_MONTHS
			};

			/*
			nobody		0, 0	0
			west		1, 0	1
			east		0, 1	2
			everybody	1, 1	3
			*/
			if (_idc == 101) then {
				if (_name_to_find == "resistanceEast") then {
					if (_value == 1) then {
						_value = (lbCurSel _idc) + 2; //plus two to the current so it ends in the lower half of the table
					} else {
						_value = lbCurSel _idc; //don't change because correct selection was already set by "resistanceWest"
					}
				}
			};

			lbSetCurSel [_idc, _value];
		};
		
		// Update text input
		if (ME3D_INTEL_IDC select _i in [104,105,106,107]) then {
			ctrlSetText [_idc, Format ["%1", _value]]
		};
		
		// Update sliders
		if (_idc in [108,109,110,111]) then {
			if (_idc in [108,110]) then {_value=1-_value};
			sliderSetPosition [_idc, _value] 
		};
	};
};

// Fill day selection list based on selected month
ME3D_EDIT_INTEL_UPDATE_DAYS_IN_MONTHS = {
	private ["_j", "_days_in_months"];
	
	if (_this >= 0) then {
		_days_in_months = [31,28,31, 30,31,30, 31,31,30, 31,30,31];
		_j = 0;
		lbClear 103;
		
		while "_j=_j+1; _j<=(_days_in_months select _this)" do {
			lbAdd [103, Format ["%1", _j]]
		};	
	};
};

// Change time to match what user selected in the Intel
ME3D_EDIT_INTEL_MATCH_TIME = {
	private ["_new_month", "_new_day", "_current_date", "_new_date", "_hour", "_minute"];
	
	_new_month = (lbCurSel 102) + 1;
	_new_day   = call (lbText [103, lbCurSel 103]);
	
	_current_date = [ME3D_CURRENT_YEAR, ME3D_CURRENT_MONTH, ME3D_CURRENT_DAY, 0, 00,00,00,000, 0, false];
	_new_date     = [ME3D_CURRENT_YEAR, _new_month        , _new_day        , 0, 00,00,00,000, 0, false];

	skipTime (([_new_date, _current_date, "subtract"] call FLIB_DATEDIFFDAY) * 24);
	
	ME3D_CURRENT_MONTH = _new_month;
	ME3D_CURRENT_DAY   = _new_day;
	
	_hour   = call (ctrlText 104);
	_minute = (call (ctrlText 105)) / 60;

	skipTime ((_hour + _minute) - daytime);
};

// Write mission settings to the mission.sqm file
ME3D_EDIT_INTEL_SAVE = {
	private ["_i", "_ok", "_idc", "_value", "_name", "_quote", "_command"];
	_i   = -1;
	_ok  = [];
	
	while "_i=_i+1; _i<count ME3D_INTEL_IDC" do {
		_idc   = ME3D_INTEL_IDC select _i;
		_value = 0;
		_quote = "";
		_name  = ME3D_INTEL_VARS select _i;
		
		// if text edit
		if (_idc in [104,105,106,107]) then {
			_value = ctrlText _idc;
			_quote = """";
			
			if (_idc in [104,105]) then {
				_quote = "";
				_value = call _value;
			};
		};
		
		// if sliders
		if (_idc in [108,109,110,111]) then {
			_value = sliderPosition _idc;
			if (_idc in [108,110]) then {_value=1-_value};
		};
		
		// if list
		if (ME3D_INTEL_IDC select _i in [101,102,103]) then {
			_value = (lbCurSel _idc) + 1;
			
			if (_name == "resistanceEast") then {
				_value = if (lbCurSel _idc >= 2) then {1} else {0};
			};
			
			if (_name == "resistanceWest") then {
				_value = if (lbCurSel _idc % 2 > 0) then {1} else {0};
			};
		};
				
		// save if changed
		if (_value != (ME3D_INTEL_CURRENT select _i)) then {
			
			// change or remove altogether if default
			_command = 
			if (_value != (ME3D_INTEL_DEFAULTS select _i)) then {
				// Wrap user input in extra quotation marks
				if (_idc in [106,107])  then {
					_value = loadFile ("\:STRING REPLACE  text:"+_value+"find:"""+"replace:""""");
				} else {
					_value = Format ["%1", _value];
				};
				
				("merge:" + _name + "=" + _quote + _value + _quote + ";")
			} else {
				Format ["deleteproperty:%1", _name]
			};
			
			_ok = call loadFile ("\:CLASS WRITE  file:mission.sqm  path:[Mission,Intel] " + _command);
			if (_ok select 0) then {
				titleText ["\n\n\n\nSaved", "PLAIN DOWN", 0.03];
			} else {
				player groupChat Format ["%1", _ok select 3];
			}
		};				
	};
};

// Check if text input is a number within given range, if not then revert value
ME3D_EDIT_INTEL_VERIFY_NUMBER = {
	private ["_idc", "_var_name", "_range_min", "_range_max", "_new_value", "_old_value", "_passed", "_type", "_number"];
	
	_idc       = _this select 0;
	_var_name  = _this select 1;
	_range_min = _this select 2;
	_range_max = _this select 3;
	
	_new_value = ctrlText _idc;
	_old_value = call (_var_name);
	_passed    = false;

	if (_new_value != _old_value) then {
		_type  = loadFile Format ["\:STRING ISNUMBER  text:%1", _new_value];

		if (_type == "integer") then {
			_number = call _new_value;
			_passed = _number >= _range_min && _number <= _range_max;
		};
		
		if (_passed) then {
			call (Format ["%1=""%2""", _var_name, _number]);
			call ME3D_EDIT_INTEL_MATCH_TIME;
		} else {
			ctrlSetText [_idc, _old_value];
		};
	};
};

// Read unit properties from the mission.sqm and display them in a dialog
ME3D_EDIT_UNIT_REFRESH = {
	private ["_side", "_group_id", "_unit_id", "_classpath", "_ok", "_i", "_property", "_idc", "_index", "_value", "_valuelist", "_valuelist_names", "_j", "_max", "_class"];
	
	_side     = _this select 0;
	_group_id = _this select 1;
	_unit_id  = _this select 2;

	
	
	// Read unit properties from the mission.sqm
	_classpath = Format ["Mission,Groups,Item%1,Vehicles,Item%2", _group_id, _unit_id];
	if (_side == "EMPTY") then {
		_classpath = Format ["Mission,Vehicles,Item%1", _unit_id];
	};
	
	_output_property0 = [];
	_output_value0    = [];
	_ok = call loadFile Format ["\:CLASS READ  file:mission.sqm  path:%1  wrap:no", _classpath];
	
	
	
	// Find unit class
	_index = ["vehicle", _output_property0] call ME3D_FIND;
	_class = if (_index >= 0) then {
		_output_value0 select _index
	} else {
		(if (_side == "EMPTY") then {
			typeOf (ME3D_VEHICLES_OBJ select _unit_id)
		} else {
			typeOf ((ME3D_GROUPS_OBJ select _group_id) select _unit_id)
		});
	};
	
	_class        = loadFile Format ["\:string case lower:1 text:%1",_class];
	_index_global = [_class, ME3D_OBJLIST_class] call ME3D_FIND;
	

	
	
	
	// Set dialog controls to read properties
	if (_ok select 0) then {
		_i                    = -1;
		ME3D_EDITUNIT_CURRENT = [];
		
		while "_i=_i+1; _i<count ME3D_EDITUNIT_PROPERTIES" do {
			_property = ME3D_EDITUNIT_PROPERTIES select _i;
			_idc      = ME3D_EDITUNIT_IDC select _i;
			_index    = [_property, _output_property0] call ME3D_FIND;
			_value    = if (_index >= 0) then {_output_value0 select _index} else {ME3D_EDITUNIT_DEFAULT select _i};
		
			// Save original value for later comparison when saving
			ME3D_EDITUNIT_CURRENT set [_i, _value];
		
			// Fill the listbox with possible values
			if (_idc in [102, 104, 105, 112, 113, 119]) then {
				_valuelist       = call Format ["ME3D_EDITUNIT_%1", _property];
				_valuelist_names = call Format ["ME3D_EDITUNIT_%1_NAME", _property];
				lbClear _idc;
				_j = 0;
				
				// Filter control list based on available vehicle seats
				if (_idc == 105) then {
					// for empty vehicles
					if (_side == "EMPTY") then {
						_valuelist       = [ME3D_EDITUNIT_PLAYER select 0];
						_valuelist_names = [ME3D_EDITUNIT_PLAYER_NAME select 0];
						"lbSetData [_idc, lbAdd [_idc, (_valuelist_names select _j) call ME3D_LOCALIZE], _x]; _j=_j+1" forEach _valuelist;
					} else {
						// if list of objects from addons has been made
						if (_index_global>=0) then {
							// for soldiers
							if ((ME3D_OBJLIST_SIMULATION select _index_global)==ME3D_SIMULATION_SOLDIER) then {
								_valuelist       = ME3D_EDITUNIT_PLAYER2;
								_valuelist_names = ME3D_EDITUNIT_PLAYER2_NAME;							
								"lbSetData [_idc, lbAdd [_idc, (_valuelist_names select _j) call ME3D_LOCALIZE], _x]; _j=_j+1" forEach _valuelist;
							} else {
								// for manned vehicles
								if ((ME3D_EDITUNIT_PLAYER_VALUE select _j) <= (ME3D_OBJLIST_SLOT select _index_global)) then {
									{lbSetData [_idc, lbAdd [_idc, (_valuelist_names select _j) call ME3D_LOCALIZE], _x]; _j=_j+1} forEach _valuelist;
								};
							};
						} else {
							// If this is player
							if (ME3D_CAMERA_TARGET == player) then {
								_valuelist       = ME3D_EDITUNIT_PLAYER2;
								_valuelist_names = ME3D_EDITUNIT_PLAYER2_NAME;							
							};
							
							"lbSetData [_idc, lbAdd [_idc, (_valuelist_names select _j) call ME3D_LOCALIZE], _x]; _j=_j+1" forEach _valuelist;
						};
					};
				} else {
					"lbSetData [_idc, lbAdd [_idc, (_valuelist_names select _j) call ME3D_LOCALIZE], _x]; _j=_j+1" forEach _valuelist;
				};
				
				// Find the value in the list of possible values
				lbSetCurSel [_idc, [_value, _valuelist] call ME3D_FIND];
			};

			// Text inputs
			if (_idc in [118, 120, 117, 115, 111]) then {
				ctrlSetText [_idc, Format ["%1",_value]];
			};
			
			// Sliders
			if (_idc in [121, 108, 109, 110, 116]) then {
				sliderSetRange [_idc, 0, 1];
				sliderSetPosition [_idc, _value];
			};
		}
	} else {
		player groupChat Format ["%1", _ok select 3];
	};


	// Handle available units and groups
	_i = 0;
	{
		//player globalchat format ["%1 %2 %3", lbCurSel 102, ((ME3D_OBJLIST_GROUP_COUNT select (lbCurSel 102)) select _i), _x];
		if (((ME3D_OBJLIST_GROUP_COUNT select (lbCurSel 102)) select _i) > 0) then {
			lbAdd [107, _x];
		};
		_i = _i + 1
	} foreach ME3D_LIST_VEHICLECLASS;
	
	if (_index_global >= 0) then {
		_category_id = ME3D_OBJLIST_VEHICLECLASS select _index_global;
		
		_i=-1; while "_i=_i+1; _i<lbSize 107" do {
			if (lbText [107, _i] == (ME3D_LIST_VEHICLECLASS select _category_id)) then {
				lbSetCurSel [107, _i];
				_vehiclegroup_sel = _i;
			}
		};
		
		_i = 0;
		{
			if (_x == _category_id  &&   ((ME3D_OBJLIST_side select _i) == lbCurSel 102 || _side == "EMPTY" && (ME3D_OBJLIST_SIMULATION select _i)!=ME3D_SIMULATION_SOLDIER)) then {
				lbSetData [103, lbAdd [103, ME3D_OBJLIST_name select _i], ME3D_OBJLIST_class select _i];
				if ((ME3D_OBJLIST_class select _i) == _class) then {
					lbSetCurSel [103, (lbSize 103)-1];
					_vehicleclass_sel = (lbSize 103)-1;
				}
			};
			_i = _i + 1;
		} forEach ME3D_OBJLIST_VEHICLECLASS;
	};
};

// Write unit properties to the mission.sqm file
ME3D_EDIT_UNIT_SAVE = {
	private ["_i", "_ok", "_side", "_group_id", "_unit_id", "_idc", "_value", "_name", "_quote", "_command", "_classpath", "_save", "_type"];
	_i   = -1;
	_ok  = [];
	
	_side     = _this select 0;
	_group_id = _this select 1;
	_unit_id  = _this select 2;
	
	while "_i=_i+1; _i<count ME3D_EDITUNIT_IDC" do {
		_idc   = ME3D_EDITUNIT_IDC select _i;
		_value = "";
		_quote = "";
		_name  = ME3D_EDITUNIT_PROPERTIES select _i;
		
		// if text edit
		if (_idc in [118, 120, 117, 115, 111]) then {
			_value = ctrlText _idc;
			_quote = """";
			
			if (_idc in [111, 115]) then {
				_quote = "";
			}
		};
		
		// if sliders
		if (_idc in [121, 108, 109, 110, 116]) then {
			_value = sliderPosition _idc;
		};
		
		// if list
		if (_idc in [102, 104, 105, 112, 113, 119]) then {
			_quote = """";
			_value = lbData [_idc, (lbCurSel _idc)];
		};
		
		_save = false;

		// condition to save
		if (Format ["%1",_value] != Format ["%1",ME3D_EDITUNIT_CURRENT select _i]) then {
			_save = true;
			
			// Verify number
			if (_idc in [111, 115]) then {
				_type = (loadFile Format ["\:STRING ISNUMBER  text:%1", _value]);

				if (_type == "string") then {
					_save  = false;
					_value = Format ["%1", ME3D_EDITUNIT_CURRENT select _i];
					ctrlSetText [_idc, _value];
				};
				
				_value = call _value;
				
				if (_idc == 111) then {
					ME3D_CAMERA_TARGET setDir _value;
					[_side, _group_id, _unit_id, (getPos ME3D_CAMERA_TARGET select 0), (getPos ME3D_CAMERA_TARGET select 1), 0, true] call ME3D_SETPOS;
				}
				
			};

			if (_idc == 108) then {
				ME3D_CAMERA_TARGET setDammage (1 - _value)
			};
			
			if (_idc == 109) then {
				ME3D_CAMERA_TARGET setFuel _value;
			};
			
			if (_idc == 121) then {
				ME3D_CAMERA_TARGET setSkill _value;
			};

			if (_idc == 119) then {
				ME3D_CAMERA_TARGET lock (_value=="LOCKED")
			};
			
			// Verify variable
			if (_idc == 118) then {
				if (!call loadFile format ["\:STRING ISVARIABLE  mode:onlyGlobal  text:%1", _value]) then {
					_save  = false;
					_value = Format ["%1", ME3D_EDITUNIT_CURRENT select _i];
					ctrlSetText [_idc, _value];
				}
			};
			
			// Delay saving
			if (_save && _idc in [121, 108, 109, 110, 116, 118, 111, 117, 115, 120]) then {
				_save = false;
				ME3D_EDITUNIT_REFRESHTIME set [_i, _time + 0.5];
			};
			
			ME3D_EDITUNIT_CURRENT set [_i, _value];
		} else {
			if (_idc in [121, 108, 109, 110, 116, 118, 111, 117, 115, 120]) then {
				if ((ME3D_EDITUNIT_REFRESHTIME select _i)>0  &&  (ME3D_EDITUNIT_REFRESHTIME select _i) < _time) then {
					ME3D_EDITUNIT_REFRESHTIME set [_i, 0];
					_save = true;
				}
			};
			
			if (_idc in [111, 115]) then {
				_value = call _value;
			}
		};

		// write to sqm file
		if (_save) then {
			// change or remove altogether if default
			_command = 
			if (_value != (ME3D_EDITUNIT_DEFAULT select _i)) then {
				// Wrap user input in extra quotation marks
				if (_idc in [120,117,118])  then {
					_value = loadFile ("\:STRING REPLACE  text:"+_value+"find:"""+"replace:""""");
				} else {
					_value = Format ["%1", _value];
				};
				
				("merge:" + _name + "=" + _quote + _value + _quote + ";")
			} else {
				Format ["deleteproperty:%1", _name]
			};
			
			_classpath = Format ["Mission,Groups,Item%1,Vehicles,Item%2", _group_id, _unit_id];
			if (_side == "EMPTY") then {
				_classpath = Format ["Mission,Vehicles,Item%1", _unit_id];
			};
			
			_ok = call loadFile (Format ["\:CLASS WRITE  file:mission.sqm  path:%1 %2 ", _classpath, ME3D_THISSETPOS] + _command);
			if (_ok select 0) then {
				titleText ["\n\n\n\nSaved", "PLAIN DOWN", 0.03];
			} else {
				player groupChat Format ["%1", _ok select 3];
			};
			
			// Refresh init field
			_output_value0 = [];
			_ok            = call loadFile Format ["\:CLASS READ  file:mission.sqm  path:%1  find:init  wrap:no", _classpath];
			
			if (count _output_value0 > 0) then {
				ctrlSetText [120, _output_value0 select 0];
				ME3D_EDITUNIT_CURRENT set [5, _output_value0 select 0];
			}
		};
	};
	
	// Get position input
	_idc_pos     = [200, 201, 202, 203];
	_refresh_pos = 0;
	_type        = lbValue [203, _input_t];
	
	_index = 0;
	{
		if ((ctrlText _x) != (_input_pos select _index)) then {
			_input_pos set [_index, (ctrlText _x)];
			_refresh_pos = if (_index==2) then {2} else {1};
		};
		
		_index = _index + 1;
	} forEach _idc_pos;
	
	// Changing height type
	if (lbCurSel 203 != _input_t) then {
		_input_t     = lbCurSel 203; 
		_type        = lbValue [203, _input_t];
		_refresh_pos = 2;
		
		if (_type == 2) then {
			_input_pos set [2, Format ["%1",getPosASL ME3D_CAMERA_TARGET select 2]];
		} else {
			_input_pos set [2, Format ["%1",getPos ME3D_CAMERA_TARGET select 2]];
		};
		
		ctrlSetText [202, _input_pos select 2];
	};
	
	// Set new position
	if (_refresh_pos > 0) then {
		_temp_pos = [];
		{_temp_pos set [count _temp_pos, call _x]} forEach _input_pos;

		if (_side != "EMPTY") then {
			(ME3D_GROUPS_Z      select _group_id) set [_unit_id, _temp_pos select 2];
			(ME3D_GROUPS_Z_TYPE select _group_id) set [_unit_id, _type];
		} else {
			ME3D_VEHICLES_Z      set [_unit_id, _temp_pos select 2];
			ME3D_VEHICLES_Z_TYPE set [_unit_id, _type];
		};

		[_side, _group_id, _unit_id, _temp_pos select 0, _temp_pos select 1, _temp_pos select 2, false] call ME3D_SETPOS;
		[_side, _group_id, _unit_id, 0] call ME3D_SAVE_SQM;
		
		if (_refresh_pos == 2) then {
			if (_side in ["WEST", "EAST", "GUER", "CIV", "LOGIC"]) then {
				_classpath = Format ["Mission,Groups,Item%1,Vehicles,Item%2", _group_id, _unit_id];
			};

			if (_side == "EMPTY") then {
				_classpath = Format ["Mission,Vehicles,Item%1", _unit_id];
			};
			
			// Refresh init field
			_output_value0 = [];
			_ok            = call loadFile Format ["\:CLASS READ  file:mission.sqm  path:%1  find:init  wrap:no", _classpath];
			
			if (count _output_value0 > 0) then {
				ctrlSetText [120, _output_value0 select 0];
				ME3D_EDITUNIT_CURRENT set [5, _output_value0 select 0];
			}
		}
	};
	
	
	// User selected a different side
	if (lbCurSel 102 != _vehicleside_sel) then {
		_vehicleside_sel  = lbCurSel 102;
		lbClear 107;
		_vehiclegroup_sel = -1;
		
		_i = 0;
		{
			//player globalchat format ["%1 %2 %3", lbCurSel 102, ((ME3D_OBJLIST_GROUP_COUNT select (lbCurSel 102)) select _i), _x];
			if (((ME3D_OBJLIST_GROUP_COUNT select (lbCurSel 102)) select _i) > 0) then {
				lbAdd [107, _x];
			};
			_i = _i + 1
		} foreach ME3D_LIST_VEHICLECLASS;
	};
	
	// User selected a different group of objects
	if (lbCurSel 107 != _vehiclegroup_sel) then {
		_vehiclegroup_sel = lbCurSel 107;

		_i = 0;
		_objects = [];
		{
			if ((ME3D_LIST_VEHICLECLASS select _x) == lbText [107,lbCurSel 107]  &&   ((lbCurSel 102!=5 && (ME3D_OBJLIST_side select _i) == lbCurSel 102)  ||  (lbCurSel 102==5 && (ME3D_OBJLIST_SIMULATION select _i)!=ME3D_SIMULATION_SOLDIER))) then {
				_objects set [count _objects, [ME3D_OBJLIST_name select _i, ME3D_OBJLIST_class select _i]];
			};
			_i = _i + 1;
		} forEach ME3D_OBJLIST_VEHICLECLASS;

		lbClear 103;
		lbAdd [103, Format ["<< %1 object%2 >>", count _objects, (if (count _objects!=1) then {"s"} else {""})]];
		lbSetCurSel [103, 0];
		_vehicleclass_sel = 0;
		{lbSetData [103, lbAdd [103, _x select 0], _x select 1]} forEach _objects;
	};
	
	// User selected a different object
	if (lbCurSel 103 != _vehicleclass_sel  &&  lbCurSel 103>0) then {
		_vehicleclass_sel = lbCurSel 103;
		
		_obj = (if (_side == "EMPTY") then {
			(ME3D_VEHICLES_OBJ select _unit_id)
		} else {
			((ME3D_GROUPS_OBJ select _group_id) select _unit_id)}
		);
		
		// Update "player control" list
		_current_control = lbData [105, lbCurSel 105];
		_index_global    = [lbData [103, lbCurSel 103], ME3D_OBJLIST_class] call ME3D_FIND;
		_j               = 0;
		lbClear 105;

		// for empty vehicles
		if (_side == "EMPTY") then {
			lbSetData [105, lbAdd [105, localize (ME3D_EDITUNIT_PLAYER_NAME select 0) call ME3D_LOCALIZE], ME3D_EDITUNIT_PLAYER select 0];
		} else {
			// for soldiers
			if ((ME3D_OBJLIST_SIMULATION select _index_global) == ME3D_SIMULATION_SOLDIER) then {
				{lbSetData [105, lbAdd [105, (ME3D_EDITUNIT_PLAYER2_NAME select _j) call ME3D_LOCALIZE], _x]; _j=_j+1} forEach ME3D_EDITUNIT_PLAYER2;
			} else {
				// for manned vehicles
				{
					if ((ME3D_EDITUNIT_PLAYER_VALUE select _j) <= (ME3D_OBJLIST_SLOT select _index_global)) then {
						lbSetData [105, lbAdd [105, (ME3D_EDITUNIT_PLAYER_NAME select _j) call ME3D_LOCALIZE], _x]; 
					};
					_j = _j + 1;
				} forEach ME3D_EDITUNIT_PLAYER;
			}
		};
		
		// find current option
		_i=-1; while "_i=_i+1; _i<lbSize 105" do {
			if (lbData [105, _i] == _current_control) then {
				lbSetCurSel [105, _i];
			};
		};
		
		// if not then select last one from the list
		if (lbCurSel 105 < 0) then {
			lbSetCurSel [105, (lbSize 105) - 1];
		};
		
		// Replace object
		if (_obj != player) then {
			deleteVehicle _obj;
			_obj = (lbData [103, lbCurSel 103]) camCreate [0,0,0];
			ME3D_CAMERA_TARGET = _obj;
			[_side, _group_id, _unit_id, call (ctrlText 200), call (ctrlText 201), call (ctrlText 202), false] call ME3D_SETPOS;
			
			if (_side == "EMPTY") then {
				(ME3D_VEHICLES_OBJ set [_unit_id, _obj])
			} else {
				((ME3D_GROUPS_OBJ select _group_id) set [_unit_id,_obj])
			};
		};
		
		// Save to sqm
		_classpath = Format ["Mission,Groups,Item%1,Vehicles,Item%2", _group_id, _unit_id];
		if (_side == "EMPTY") then {
			_classpath = Format ["Mission,Vehicles,Item%1", _unit_id];
		};
		
		_ok = call loadFile (Format ["\:CLASS WRITE  file:mission.sqm  path:%1 %2 merge:vehicle=""%3"";",_classpath, ME3D_THISSETPOS, lbData [103, lbCurSel 103]]);
		if (_ok select 0) then {
			titleText ["\n\n\n\nSaved", "PLAIN DOWN", 0.03];
		} else {
			player groupChat Format ["%1", _ok select 3];
		};
		
		// Refresh initialization
		_output_value0    = [];
		_ok = call loadFile Format ["\:CLASS READ  file:mission.sqm  path:%1  find:init  wrap:no", _classpath];
		
		if (count _output_value0 > 0) then {
			ctrlSetText [120, _output_value0 select 0];
			ME3D_EDITUNIT_CURRENT set [5, _output_value0 select 0];
		}
	};
};

// Read trigger properties from the mission.sqm and display them in a dialog
ME3D_EDIT_TRIGGER_REFRESH = {
	private ["_side", "_group_id", "_unit_id", "_classpath", "_ok", "_i", "_property", "_idc", "_index", "_value", "_valuelist", "_valuelist_names", "_j", "_max"];
	
	_side     = _this select 0;
	_group_id = _this select 1;
	_unit_id  = _this select 2;
	
	_classpath        = Format ["Mission,Sensors,Item%1", _unit_id];
	_output_property0 = [];
	_output_value0    = [];
	_ok               = call loadFile Format ["\:CLASS READ  file:mission.sqm  path:%1  wrap:no", _classpath];
	
	if (_ok select 0) then {
		_i                       = -1;
		ME3D_EDITTRIGGER_CURRENT = [];
		
		while "_i=_i+1; _i<count ME3D_EDITTRIGGER_PROPERTIES" do {
			_property = ME3D_EDITTRIGGER_PROPERTIES select _i;
			_idc      = ME3D_EDITTRIGGER_IDC select _i;
			_index    = [_property, _output_property0] call ME3D_FIND;
			_value    = if (_index >= 0) then {_output_value0 select _index} else {ME3D_EDITTRIGGER_DEFAULT select _i};
		
			// Save original value for later comparison when saving
			ME3D_EDITTRIGGER_CURRENT set [_i, _value];
		
			// Fill the listbox with possible values
			if (_idc in [120,107,105,106,198,112,108]) then {
				_valuelist       = call Format ["ME3D_EDITTRIGGER_%1", _property];
				_valuelist_names = call Format ["ME3D_EDITTRIGGER_%1_NAME", _property];
				lbClear _idc;
				_j = 0;
				"lbSetData [_idc, lbAdd [_idc, (_valuelist_names select _j) call ME3D_LOCALIZE], Format [""%1"",_x]]; _j=_j+1" forEach _valuelist;
				
				// Find the value in the list of possible values
				lbSetCurSel [_idc, [_value, _valuelist] call ME3D_FIND];
			};

			// Text inputs
			if (_idc in [102,104,103,118,117,119,121,114,110,111,109]) then {
				ctrlSetText [_idc, Format ["%1",_value]];
			};
		}
	} else {
		player groupChat Format ["%1", _ok select 3];
	};
};

// Write trigger properties to the mission.sqm file
ME3D_EDIT_TRIGGER_SAVE = {
	private ["_i", "_ok", "_side", "_group_id", "_unit_id", "_idc", "_value", "_name", "_quote", "_command", "_classpath", "_save", "_type"];
	_i   = -1;
	_ok  = [];
	
	_side     = _this select 0;
	_group_id = _this select 1;
	_unit_id  = _this select 2;
	
	while "_i=_i+1; _i<count ME3D_EDITTRIGGER_IDC" do {
		_idc   = ME3D_EDITTRIGGER_IDC select _i;
		_value = "";
		_quote = "";
		_name  = ME3D_EDITTRIGGER_PROPERTIES select _i;
		
		// if text edit
		if (_idc in [102,104,103,118,117,119,121,114,110,111,109])  then {
			_value = ctrlText _idc;
			_quote = """";
			
			// numeric inputs
			if (_idc in [102,104,103,110,111,109]) then {
				_quote = "";
			}
		};
		
		// if list
		if (_idc in [120,107,105,106,198,112,108]) then {
			_quote = """";
			_value = lbData [_idc, (lbCurSel _idc)];
			
			if (_idc in [120,107,108]) then {
				_quote = "";
			};
		};
		
		_save = false;

		// condition to save
		if (Format ["%1",_value] != Format ["%1",ME3D_EDITTRIGGER_CURRENT select _i]) then {
			_save = true;
			
			// Verify number
			if (_idc in [102,104,103,110,111,109, 120,107,108]) then {
				_type = (loadFile Format ["\:STRING ISNUMBER  text:%1", _value]);

				if (_type == "string") then {
					_save  = false;
					_value = Format ["%1", ME3D_EDITTRIGGER_CURRENT select _i];
					ctrlSetText [_idc, _value];
				};
				
				_value = call _value;				
			};
			
			// Verify variable
			if (_idc == 114) then {
				if (!call loadFile format ["\:STRING ISVARIABLE  mode:onlyGlobal  text:%1", _value]) then {
					_save  = false;
					_value = Format ["%1", ME3D_EDITTRIGGER_CURRENT select _i];
					ctrlSetText [_idc, _value];
				}
			};
			
			// Delay saving
			if (_save && _idc in [102,104,103,118,117,119,121,114,110,111,109]) then {
				_save = false;
				ME3D_EDITTRIGGER_REFRESHTIME set [_i, _time + 0.5];
			};
			
			ME3D_EDITTRIGGER_CURRENT set [_i, _value];
		} else {
			if (_idc in [102,104,103,118,117,119,121,114,110,111,109]) then {
				if ((ME3D_EDITTRIGGER_REFRESHTIME select _i)>0  &&  (ME3D_EDITTRIGGER_REFRESHTIME select _i) < _time) then {
					ME3D_EDITTRIGGER_REFRESHTIME set [_i, 0];
					_save = true;
				}
			};
			
			if (_idc in [102,104,103,110,111,109, 120,107,108]) then {
				_value = call _value;
			}
		};

		// write to sqm file
		if (_save) then {
			// change or remove altogether if default
			_command = 
			if (_value != (ME3D_EDITTRIGGER_DEFAULT select _i)) then {
				// Wrap user input in extra quotation marks
				if (_idc in [118,117,119,121,114])  then {
					_value = loadFile ("\:STRING REPLACE  text:"+_value+"find:"""+"replace:""""");
				} else {
					_value = Format ["%1", _value];
				};
				
				("merge:" + _name + "=" + _quote + _value + _quote + ";")
			} else {
				Format ["deleteproperty:%1", _name]
			};
			
			_classpath = Format ["Mission,Sensors,Item%1", _unit_id];		
			_ok = call loadFile (Format ["\:CLASS WRITE  file:mission.sqm  path:%1 ",_classpath] + _command);
			if (_ok select 0) then {
				titleText ["\n\n\n\nSaved", "PLAIN DOWN", 0.03];
			} else {
				player groupChat Format ["%1", _ok select 3];
			}
		};
	};
};

// Read marker properties from the mission.sqm and display them in a dialog
ME3D_EDIT_MARKER_REFRESH = {
	private ["_side", "_group_id", "_unit_id", "_classpath", "_ok", "_i", "_property", "_idc", "_index", "_value", "_valuelist", "_valuelist_names", "_j", "_max"];
	
	_side     = _this select 0;
	_group_id = _this select 1;
	_unit_id  = _this select 2;
	
	_classpath = Format ["Mission,Markers,Item%1", _unit_id];
	
	_output_property0 = [];
	_output_value0    = [];
	_ok = call loadFile Format ["\:CLASS READ  file:mission.sqm  path:%1  wrap:no", _classpath];
	if (_ok select 0) then {
		_i                       = -1;
		ME3D_EDITMARKER_CURRENT = [];
		
		while "_i=_i+1; _i<count ME3D_EDITMARKER_PROPERTIES" do {
			_property = ME3D_EDITMARKER_PROPERTIES select _i;
			_idc      = ME3D_EDITMARKER_IDC select _i;
			_index    = [_property, _output_property0] call ME3D_FIND;
			_value    = if (_index >= 0) then {_output_value0 select _index} else {ME3D_EDITMARKER_DEFAULT select _i};
		
			// Save original value for later comparison when saving
			ME3D_EDITMARKER_CURRENT set [_i, _value];
		
			// Fill the listbox with possible values
			if (_idc in [103, 105, 104, 110]) then {
				_valuelist       = call Format ["ME3D_EDITMARKER_%1", _property];
				_valuelist_names = call Format ["ME3D_EDITMARKER_%1_NAME", _property];
				lbClear _idc;
				_j = 0;
				"lbSetData [_idc, lbAdd [_idc, _valuelist_names select _j], Format [""%1"",_x]]; _j=_j+1" forEach _valuelist;
				
				// Find the value in the list of possible values
				lbSetCurSel [_idc, [_value, _valuelist] call ME3D_FIND];
			};

			// Text inputs
			if (_idc in [106,108,107,102,111]) then {
				ctrlSetText [_idc, Format ["%1",_value]];
			};
		};
	} else {
		player groupChat Format ["%1", _ok select 3];
	};	
};

// Write marker properties to the mission.sqm file
ME3D_EDIT_MARKER_SAVE = {
	private ["_i", "_ok", "_side", "_group_id", "_unit_id", "_idc", "_value", "_name", "_quote", "_command", "_classpath", "_save", "_type"];
	_i   = -1;
	_ok  = [];
	
	_side     = _this select 0;
	_group_id = _this select 1;
	_unit_id  = _this select 2;
	
	while "_i=_i+1; _i<count ME3D_EDITMARKER_IDC" do {
		_idc   = ME3D_EDITMARKER_IDC select _i;
		_value = "";
		_quote = "";
		_name  = ME3D_EDITMARKER_PROPERTIES select _i;
		
		// if text edit
		if (_idc in [106,108,107,102,111])  then {
			_value = ctrlText _idc;
			_quote = """";
			
			// numeric inputs
			if (_idc in [106,108,107]) then {
				_quote = "";
			}
		};
		
		// if list
		if (_idc in [103, 105, 104, 110]) then {
			_quote = """";
			_value = lbData [_idc, (lbCurSel _idc)];
			
			/*if (_idc in [120,107,108]) then {
				_quote = "";
			};*/
		};
		
		_save = false;

		// condition to save
		if (Format ["%1",_value] != Format ["%1",ME3D_EDITMARKER_CURRENT select _i]) then {
			_save = true;

			// Change marker texture
			if (ME3D_ADDON_PRESENT && _idc == 104) then {
				_index = [_value, ME3D_EDITMARKER_TYPE] call ME3D_FIND;
				
				if (_index >= 0) then {
					_pos = getPos ME3D_GROUND_TEX;
					ME3D_GROUND_TEX setPos [0,0,0];
					ME3D_GROUND_TEX setObjectTexture [1, ME3D_EDITMARKER_TYPE_TEX select _index];
					ME3D_GROUND_TEX setPos _pos;
				} else {
					ME3D_GROUND_TEX setObjectTexture [1, ""];
				};
			};
			
			// Verify number
			if (_idc in [106,108,107]) then {
				_type = (loadFile Format ["\:STRING ISNUMBER  text:%1", _value]);

				if (_type == "string") then {
					_save  = false;
					_value = Format ["%1", ME3D_EDITMARKER_CURRENT select _i];
					ctrlSetText [_idc, _value];
				};
				
				_value = call _value;				
			};
			
			// Delay saving
			if (_save && _idc in [106,108,107,102,111]) then {
				_save = false;
				ME3D_EDITMARKER_REFRESHTIME set [_i, _time + 0.5];
			};
			
			ME3D_EDITMARKER_CURRENT set [_i, _value];
		} else {
			if (_idc in [106,108,107,102,111]) then {
				if ((ME3D_EDITMARKER_REFRESHTIME select _i)>0  &&  (ME3D_EDITMARKER_REFRESHTIME select _i) < _time) then {
					ME3D_EDITMARKER_REFRESHTIME set [_i, 0];
					_save = true;
				}
			};
			
			if (_idc in [106,108,107]) then {
				_value = call _value;
			}
		};

		// write to sqm file
		if (_save) then {
			// change or remove altogether if default
			_command = 
			if (_value != (ME3D_EDITMARKER_DEFAULT select _i)) then {
				// Wrap user input in extra quotation marks
				if (_idc in [102,111])  then {
					_value = loadFile ("\:STRING REPLACE  text:"+_value+"find:"""+"replace:""""");
				} else {
					_value = Format ["%1", _value];
				};
				
				("merge:" + _name + "=" + _quote + _value + _quote + ";")
			} else {
				Format ["deleteproperty:%1", _name]
			};
			
			_classpath = Format ["Mission,Markers,Item%1", _unit_id];		
			_ok = call loadFile (Format ["\:CLASS WRITE  file:mission.sqm  path:%1 ",_classpath] + _command);
			if (_ok select 0) then {
				titleText ["\n\n\n\nSaved", "PLAIN DOWN", 0.03];
			} else {
				player groupChat Format ["%1", _ok select 3];
			}
		};
	};
};

// Read waypoint properties from the mission.sqm and display them in a dialog
ME3D_EDIT_WAYPOINT_REFRESH = {
	private ["_side", "_group_id", "_waypoint_id", "_classpath", "_ok", "_i", "_property", "_idc", "_index", "_value", "_valuelist", "_valuelist_names", "_j", "_max"];
	
	_side         = _this select 0;
	_group_id     = _this select 1;
	_waypoint_id  = _this select 2;
	
	_classpath = Format ["[Mission,Groups,Item%1,Waypoints,Item%2]", _group_id, _waypoint_id];
	
	_output_property0 = [];
	_output_value0    = [];
	_ok = call loadFile Format ["\:CLASS READ  file:mission.sqm  path:%1  wrap:no", _classpath];

	if (_ok select 0) then {
		_i                       = -1;
		ME3D_EDITWAYPOINT_CURRENT = [];
		
		while "_i=_i+1; _i<count ME3D_EDITWAYPOINT_PROPERTIES" do {
			_property = ME3D_EDITWAYPOINT_PROPERTIES select _i;
			_idc      = ME3D_EDITWAYPOINT_IDC select _i;
			_index    = [_property, _output_property0] call ME3D_FIND;
			_value    = if (_index >= 0) then {_output_value0 select _index} else {ME3D_EDITWAYPOINT_DEFAULT select _i};
		
			// Save original value for later comparison when saving
			ME3D_EDITWAYPOINT_CURRENT set [_i, _value];
		
			// Fill the listbox with possible values
			if (_idc in [102,105,106,107,108,117]) then {
				_valuelist       = call Format ["ME3D_EDITWAYPOINT_%1", _property];
				_valuelist_names = call Format ["ME3D_EDITWAYPOINT_%1_NAME", _property];
				lbClear _idc;
				_j = 0;
				"lbSetData [_idc, lbAdd [_idc, (_valuelist_names select _j) call ME3D_LOCALIZE], Format [""%1"",_x]]; _j=_j+1" forEach _valuelist;
				
				// Find the value in the list of possible values
				lbSetCurSel [_idc, [_value, _valuelist] call ME3D_FIND];
			};

			// Text inputs
			if (_idc in [104,109,112,113,111,116,118,119]) then {
				ctrlSetText [_idc, Format ["%1",_value]];
			};
		};
	} else {
		player groupChat Format ["%1", _ok select 3];
	};

	// waypoint order
	lbClear 103;
	_i = 0;
	{
		lbAdd [103, if (_i != _waypoint_id) then {Format ["%1: place it before %2", _i, _x select 1]} else {Format ["%1: %2", _i, _x select 1]}];
		_i = _i + 1;
	} forEach (ME3D_GROUPS_WP select _group_id);
	
	lbAdd [103, Format ["%1: insert at the end", count (ME3D_GROUPS_WP select _group_id)]];
	lbSetCurSel [103, _waypoint_id];
};

// Write waypoint properties to the mission.sqm file
ME3D_EDIT_WAYPOINT_SAVE = {
	private ["_i", "_ok", "_side", "_group_id", "_unit_id", "_idc", "_value", "_name", "_quote", "_command", "_classpath", "_save", "_type"];
	_i   = -1;
	_ok  = [];
	
	_side     = _this select 0;
	_group_id = _this select 1;
	_unit_id  = _this select 2;
	
	while "_i=_i+1; _i<count ME3D_EDITWAYPOINT_IDC" do {
		_idc   = ME3D_EDITWAYPOINT_IDC select _i;
		_value = "";
		_quote = "";
		_name  = ME3D_EDITWAYPOINT_PROPERTIES select _i;
		
		// if text edit
		if (_idc in [104,109,112,113,111,116,118,119])  then {
			_value = ctrlText _idc;
			_quote = """";
			
			// numeric inputs
			if (_idc in [109,112,113,111]) then {
				_quote = "";
			}
		};
		
		// if list
		if (_idc in [102,105,106,107,108,117]) then {
			_quote = """";
			_value = lbData [_idc, (lbCurSel _idc)];
		};
		
		_save = false;

		// condition to save
		if (Format ["%1",_value] != Format ["%1",ME3D_EDITWAYPOINT_CURRENT select _i]) then {
			_save = true;
			
			// Verify number
			if (_idc in [109,112,113,111]) then {
				_type = (loadFile Format ["\:STRING ISNUMBER  text:%1", _value]);

				if (_type == "string") then {
					_save  = false;
					_value = Format ["%1", ME3D_EDITWAYPOINT_CURRENT select _i];
					ctrlSetText [_idc, _value];
				};
				
				_value = call _value;				
			};
			
			// Delay saving
			if (_save && _idc in [104,109,112,113,111,116,118,119]) then {
				_save = false;
				ME3D_EDITWAYPOINT_REFRESHTIME set [_i, _time + 0.5];
			};
			
			ME3D_EDITWAYPOINT_CURRENT set [_i, _value];
		} else {
			if (_idc in [104,109,112,113,111,116,118,119]) then {
				if ((ME3D_EDITWAYPOINT_REFRESHTIME select _i)>0  &&  (ME3D_EDITWAYPOINT_REFRESHTIME select _i) < _time) then {
					ME3D_EDITWAYPOINT_REFRESHTIME set [_i, 0];
					_save = true;
				}
			};
			
			if (_idc in [109,112,113,111]) then {
				_value = call _value;
			}
		};

		// write to sqm file
		if (_save) then {
			// change or remove altogether if default
			_command = 
			if (_value != (ME3D_EDITWAYPOINT_DEFAULT select _i)) then {
				// Wrap user input in extra quotation marks
				if (_idc in [104,116,118,119])  then {
					_value = loadFile ("\:STRING REPLACE  text:"+_value+"find:"""+"replace:""""");
				} else {
					_value = Format ["%1", _value];
				};
				
				("merge:" + _name + "=" + _quote + _value + _quote + ";")
			} else {
				Format ["deleteproperty:%1", _name]
			};
			
			_classpath = Format ["[Mission,Groups,Item%1,Waypoints,Item%2]", _group_id, _waypoint_id];
			_ok = call loadFile (Format ["\:CLASS WRITE  file:mission.sqm  path:%1 ",_classpath] + _command);
			if (_ok select 0) then {
				titleText ["\n\n\n\nSaved", "PLAIN DOWN", 0.03];
			} else {
				player groupChat Format ["%1", _ok select 3];
			}
		};
	};
	
	// Change waypoint order
	if (lbCurSel 103 != _waypoint_id) then {
		if (lbCurSel 103 > _waypoint_id) then {
			// moving forward
			_destination_id = (lbCurSel 103) - 1;
			
			_ok = call loadFile Format ["\:CLASS WRITE  file:mission.sqm  path:Mission,Groups,Item%1,Waypoints  renameclass:Item%2  to:ItemTEMP", _group_id, _waypoint_id];
			_temp =+ (ME3D_GROUPS_WP select _group_id) select _waypoint_id;
			
			if (_ok select 0) then {
				_i = _waypoint_id + 1;
				while "_i <= _destination_id" do {
					loadFile Format ["\:CLASS WRITE  file:mission.sqm  path:Mission,Groups,Item%1,Waypoints  renameclass:Item%2  to:Item%3", _group_id, _i, _i-1];
					(ME3D_GROUPS_WP select _group_id) set [_i-1, (ME3D_GROUPS_WP select _group_id) select _i];
					_i = _i + 1;
				};
				
				loadFile Format ["\:CLASS WRITE  file:mission.sqm  path:Mission,Groups,Item%1,Waypoints  renameclass:ItemTEMP  to:Item%2", _group_id, _destination_id];
				(ME3D_GROUPS_WP select _group_id) set [_destination_id, _temp];
			} else {
				player groupChat Format ["%1", _ok select 3];
			};
			
			_waypoint_id = _destination_id;
			[_side, _group_id, _waypoint_id] call ME3D_EDIT_WAYPOINT_REFRESH;
		} else {
			// moving backward
			_destination_id = (lbCurSel 103);
			
			_ok = call loadFile Format ["\:CLASS WRITE  file:mission.sqm  path:Mission,Groups,Item%1,Waypoints  renameclass:Item%2  to:ItemTEMP", _group_id, _waypoint_id];
			if (_ok select 0) then {
				_i = _destination_id;
				while "_i < _waypoint_id" do {
					loadFile Format ["\:CLASS WRITE  file:mission.sqm  path:Mission,Groups,Item%1,Waypoints  renameclass:Item%2  to:Item%3", _group_id, _i, _i+1];
					_i = _i + 1;
				};
				
				loadFile Format ["\:CLASS WRITE  file:mission.sqm  path:Mission,Groups,Item%1,Waypoints  renameclass:ItemTEMP  to:Item%2", _group_id, _destination_id];
			} else {
				player groupChat Format ["%1", _ok select 3];
			};
			
			_waypoint_id = _destination_id;
			[_side, _group_id, _waypoint_id] call ME3D_EDIT_WAYPOINT_REFRESH;
		};
	};
};

// Read effects properties from the mission.sqm and display them in a dialog
ME3D_EDIT_EFFECTS_REFRESH = {
	private ["_side", "_group_id", "_unit_id", "_classpath", "_ok", "_i", "_property", "_idc", "_index", "_value", "_valuelist", "_valuelist_names", "_j", "_max"];
	
	_side     = _this select 0;
	_group_id = _this select 1;
	_unit_id  = _this select 2;
	
	_classpath = if (_side == "TRIGGER") then {
		Format ["[Mission,Sensors,Item%2,Effects]", _group_id, _unit_id]
	} else {
		Format ["[Mission,Groups,Item%1,Waypoints,Item%2,Effects]", _group_id, _unit_id]
	};
	
	_output_property0 = [];
	_output_value0    = [];
	_ok = call loadFile Format ["\:CLASS READ  file:mission.sqm  path:%1  wrap:no", _classpath];

	if (_ok select 0) then {
		_i                       = -1;
		ME3D_EDITEFFECTS_CURRENT = [];
		
		while "_i=_i+1; _i<count ME3D_EDITEFFECTS_PROPERTIES" do {
			_property = ME3D_EDITEFFECTS_PROPERTIES select _i;
			_idc      = ME3D_EDITEFFECTS_IDC select _i;
			_index    = [_property, _output_property0] call ME3D_FIND;
			_value    = if (_index >= 0) then {_output_value0 select _index} else {ME3D_EDITEFFECTS_DEFAULT select _i};
		
			// Save original value for later comparison when saving
			ME3D_EDITEFFECTS_CURRENT set [_i, _value];
		
			// Fill the listbox with possible values
			if (_idc in [101, 102, 103, 104, 105, 106, 107, 108, 109]) then {
				_valuelist       = call Format ["ME3D_EDITEFFECTS_%1", _property];
				_valuelist_names = call Format ["ME3D_EDITEFFECTS_%1_NAME", _property];
				
				if (_idc == 104) then {
					_valuelist       = ME3D_EDITEFFECTS_SOUND;
					_valuelist_names = ME3D_EDITEFFECTS_SOUND_NAME;
				};
							
				if (_idc == 112) then {
					_valuelist       = ME3D_CfgTitles;
					_valuelist_names = ME3D_CfgTitles_NAME;
				};
				
				lbClear _idc;
				_j = 0;
				"lbSetData [_idc, lbAdd [_idc, (_valuelist_names select _j) call ME3D_LOCALIZE], Format [""%1"",_x]]; _j=_j+1" forEach _valuelist;
				
				// Find the value in the list of possible values
				lbSetCurSel [_idc, [_value, _valuelist] call ME3D_FIND];
			};

			// Text inputs
			if (_idc in [113, 110]) then {
				ctrlSetText [_idc, Format ["%1",_value]];
			};
		};
	} else {
		player groupChat Format ["%1", _ok select 3];
	};
	
	_i = 0;
	"lbSetData [112, lbAdd [112, (ME3D_CfgTitles_NAME select _i) call ME3D_LOCALIZE], Format [""%1"",_x]]; _i=_i+1" forEach ME3D_CfgTitles;
	lbSetCurSel [112, 0];
	
	_i = 0;
	"lbSetData [111, lbAdd [111, (ME3D_RscTitles_NAME select _i)], Format [""%1"",_x]]; _i=_i+1" forEach ME3D_RscTitles;
	lbSetCurSel [111, 0];
	
	_title_type_selection = lbCurSel 108;
	"ctrlShow [_x,false]" forEach [110, 111, 112];
	ctrlShow [[0,112,111,110] select _title_type_selection, true];
	
	_index = ["title", _output_property0] call ME3D_FIND;
	
	if (_title_type_selection == 3) then {
		ctrlSetText [110, (if (_index >= 0) then {_titletext_input=_output_value0 select _index; _titletext_input} else {""})]
	} else {
		_array_value = [];
		_array_name  = [];
		
		if (_title_type_selection == 1) then {
			_array_value = ME3D_CfgTitles; 
			_array_name  = ME3D_CfgTitles_NAME;
		};
		
		if (_title_type_selection == 2) then {
			_array_value = ME3D_RscTitles; 
			_array_name  = ME3D_RscTitles_NAME;
		};
			
		if (_index >= 0) then {
			_value = _output_value0 select _index;
			_index = [_value, _array_value] call ME3D_FIND;
			
			if (_title_type_selection == 1) then {
				_object_selection = _index;
			};
			
			if (_title_type_selection == 2) then {
				_resource_selection = _index;
			};
			
			if (_index >= 0) then {
				lbSetCurSel [[0,112,111,110] select _title_type_selection, _index];
			} else {
				if (_title_type_selection in [1,2]) then {
					lbSetCurSel [[0,112,111,110] select _title_type_selection, 0];
				}
			}
		}
	}
};

// Write effects properties to the mission.sqm file
ME3D_EDIT_EFFECTS_SAVE = {
	private ["_i", "_ok", "_side", "_group_id", "_unit_id", "_idc", "_value", "_name", "_quote", "_command", "_classpath", "_save", "_type"];
	_i   = -1;
	_ok  = [];
	
	_side     = _this select 0;
	_group_id = _this select 1;
	_unit_id  = _this select 2;
	
	while "_i=_i+1; _i<count ME3D_EDITEFFECTS_IDC" do {
		_idc   = ME3D_EDITEFFECTS_IDC select _i;
		_value = "";
		_quote = "";
		_name  = ME3D_EDITEFFECTS_PROPERTIES select _i;
		
		// if text edit
		if (_idc in [113, 110])  then {
			_value = ctrlText _idc;
			_quote = """";
		};
		
		// if list
		if (_idc in [101, 102, 103, 104, 105, 106, 107, 108, 109]) then {
			_quote = """";
			_value = lbData [_idc, (lbCurSel _idc)];
		};
		
		_save = false;

		// condition to save
		if (Format ["%1",_value] != Format ["%1",ME3D_EDITEFFECTS_CURRENT select _i]) then {
			_save = true;
			
			// Delay saving
			if (_save && _idc in [113, 110]) then {
				_save = false;
				ME3D_EDITEFFECTS_REFRESHTIME set [_i, _time + 0.5];
			};
			
			ME3D_EDITEFFECTS_CURRENT set [_i, _value];

			if (ME3D_ENABLE_PREVIEW) then {
				if (_idc in [103, 104]) then {	// play cfgSounds
					playSound _value;
				};
				
				if (_idc == 107) then {	// player cfgMusic
					playMusic _value;
				};
			};
			
			// Force preview
			if (_idc == 109) then {
				if (_title_type_selection == 1) then {_object_selection=-1};
				if (_title_type_selection == 2) then {_resource_selection=-1};
				if (_title_type_selection == 3) then {_titletext_input=""};
			};
		} else {
			if (_idc in [113, 110]) then {
				if ((ME3D_EDITEFFECTS_REFRESHTIME select _i)>0  &&  (ME3D_EDITEFFECTS_REFRESHTIME select _i) < _time) then {
					ME3D_EDITEFFECTS_REFRESHTIME set [_i, 0];
					_save = true;
				}
			};
		};

		// write to sqm file
		if (_save) then {
			// change or remove altogether if default
			_command = 
			if (_value != (ME3D_EDITEFFECTS_DEFAULT select _i)) then {
				// Wrap user input in extra quotation marks
				if (_idc in [113, 110])  then {
					_value = loadFile ("\:STRING REPLACE  text:"+_value+"find:"""+"replace:""""");
				} else {
					_value = Format ["%1", _value];
				};
				
				("merge:" + _name + "=" + _quote + _value + _quote + ";")
			} else {
				Format ["deleteproperty:%1", _name]
			};
			
			_classpath = if (_side == "TRIGGER") then {
				Format ["[Mission,Sensors,Item%2,Effects]", _group_id, _unit_id]
			} else {
				Format ["[Mission,Groups,Item%1,Waypoints,Item%2,Effects]", _group_id, _unit_id]
			};
			
			_ok = call loadFile (Format ["\:CLASS WRITE  file:mission.sqm  path:%1 ",_classpath] + _command);
			if (_ok select 0) then {
				titleText ["\n\n\n\nSaved", "PLAIN DOWN", 0.03];
			} else {
				player groupChat Format ["%1", _ok select 3];
			}
		};
	};
	
	_save_property = false;
	_save_value    = "";
	
	if (lbCurSel 108 != _title_type_selection) then {
		_title_type_selection = lbCurSel 108;
		
		"ctrlShow [_x,false]" forEach [110, 111, 112];
		ctrlShow [[0,112,111,110] select _title_type_selection, true];
		_save_property = true;
	};
	
	// CfgTitles
	if (_title_type_selection==1  &&  lbCurSel 112 != _object_selection) then {
		_object_selection = lbCurSel 112;
		_save_property    = true;
		_save_value       = ME3D_CfgTitles select _object_selection;
		
		if (ME3D_ENABLE_PREVIEW) then {
			cutObj [_save_value, lbData [109, (lbCurSel 109)]];
			_wait_to_release = 1;
		};
	};
	
	// RscTitles
	if (_title_type_selection==2  &&  lbCurSel 111 != _resource_selection) then {
		_resource_selection = lbCurSel 111;
		_save_property      = true;
		_save_value         = ME3D_RscTitles select _resource_selection;
		
		if (ME3D_ENABLE_PREVIEW) then {
			cutRsc [_save_value, lbData [109, (lbCurSel 109)]];
			_wait_to_release = 1;
		};
	};
	
	// Text
	if (_title_type_selection==3  &&  ctrlText 110 != _titletext_input) then {
		_titletext_input = ctrlText 110;
		_save_property   = true;
		_save_value      = _titletext_input;
		
		if (ME3D_ENABLE_PREVIEW) then {
			cutText [_save_value, lbData [109, (lbCurSel 109)]];
			_wait_to_release = 1;
		};
	};
	
	if (_save_property) then {
		_save_property = false;
		_classpath = if (_side == "TRIGGER") then {
			Format ["[Mission,Sensors,Item%2,Effects]", _group_id, _unit_id]
		} else {
			Format ["[Mission,Groups,Item%1,Waypoints,Item%2,Effects]", _group_id, _unit_id]
		};
		
		_ok = call loadFile Format ["\:CLASS WRITE  file:mission.sqm  path:%1  merge:title=""%2"";", _classpath, _save_value];
		if (_ok select 0) then {
			titleText ["\n\n\n\nSaved", "PLAIN DOWN", 0.03];
		} else {
			player groupChat Format ["%1", _ok select 3];
		}
	};
};







//---- Misc ----------------------------------------------------------------------------

// Write new position/direction to the mission.sqm file
ME3D_SAVE_SQM = {
	private ["_pos", "_dir", "_side", "_group_id", "_unit_id", "_waypoint_id", "_ok", "_classpath","_z_type","_i","_setpos","_first_part","_z_array"];
	
	_pos         = getPos ME3D_CAMERA_TARGET;
	_dir         = getDir ME3D_CAMERA_TARGET;
	_side        = lbData [1, lbCurSel 1];
	_group_id    = call (lbData [2, lbCurSel 2]);
	_unit_id     = lbCurSel 3;
	_waypoint_id = lbCurSel 4;
	_classpath   = "";
	_ok          = [];
	_z_type      = 0;

	// If being called from dialog other than "ME3D_EditUnit" then use function arguments
	if (count _this > 0) then {
		_side        = _this select 0;
		_group_id    = _this select 1;
		_unit_id     = _this select 2;
		_waypoint_id = _this select 3;
	};

	if (ME3D_CURRENT_SELECTION_TYPE == "soldier") then {
		if (_side in ["WEST", "EAST", "GUER", "CIV", "LOGIC"]) then {
			_classpath = Format ["Mission,Groups,Item%1,Vehicles,Item%2", _group_id, _unit_id];
			
			_z_type = (ME3D_GROUPS_Z_TYPE select _group_id) select _unit_id;
			
			if (_z_type == 2) then {
				_pos = getPosASL ME3D_CAMERA_TARGET;
			};
			
			_pos set [2, (ME3D_GROUPS_Z select _group_id) select _unit_id];
		};

		if (_side == "EMPTY") then {
			_classpath = Format ["Mission,Vehicles,Item%1", _unit_id];
			
			_z_type = ME3D_VEHICLES_Z_TYPE select _unit_id;
			
			if (_z_type == 2) then {
				_pos = getPosASL ME3D_CAMERA_TARGET;
			};
			
			_pos set [2, ME3D_VEHICLES_Z select _unit_id];
		};

		if (_side == "MARKER") then {
			_classpath = Format ["Mission,Markers,Item%1", _unit_id];
		};

		if (_side == "TRIGGER") then {
			_classpath = Format ["Mission,Sensors,Item%1", _unit_id];
		};
	} else {
		((ME3D_GROUPS_WP select _group_id) select _waypoint_id) set [0, [_pos select 0, _pos select 1, 0]];
		_classpath = Format ["[Mission,Groups,Item%1,Waypoints,Item%2]", _group_id, _waypoint_id];
	};

	// If object position changed then update "position[]" in the mission.sqm class
	if ((ME3D_CAMERA_TARGET_ORIGINAL_POS select 0) != (_pos select 0)  ||  (ME3D_CAMERA_TARGET_ORIGINAL_POS select 1) != (_pos select 1)  ||  (ME3D_CAMERA_TARGET_ORIGINAL_POS select 2) != (_pos select 2)  ||  ME3D_CAMERA_TARGET_ORIGINAL_DIR != _dir) then {
		if (ME3D_CAMERA_TARGET_ORIGINAL_DIR != _dir) then {
			[_side, _group_id, _unit_id, _pos select 0, _pos select 1, 0, true] call ME3D_SETPOS
		};
		
		//player groupChat Format ["position[]={%1,%2,%3};", (_pos select 0), 0, (_pos select 1)];
		_ok = call loadFile (Format ["\:CLASS WRITE  file:mission.sqm  path:%1  %2  merge:position[]=%3;", _classpath, ME3D_THISSETPOS, [(_pos select 0), 0, (_pos select 1)], _pos select 2]);
		
		if (!(_ok select 0)) then {
			player groupchat Format ["%1", _ok];
		};
		
		ME3D_CAMERA_TARGET_ORIGINAL_POS =+ _pos;
	};
	
	if (ME3D_CURRENT_SELECTION_TYPE == "soldier"  &&  ME3D_CAMERA_TARGET_ORIGINAL_DIR != _dir) then {
		 ME3D_CAMERA_TARGET_ORIGINAL_DIR = _dir;
		_ok = call loadFile Format ["\:CLASS WRITE  file:mission.sqm  path:%1  merge:%2=%3;", _classpath, (if (_side in ["MARKER","TRIGGER"]) then {"angle"} else {"azimut"}), _dir];
		
		if (!(_ok select 0)) then {
			player groupchat Format ["%1", _ok];
		}
	};
};

// do setPos or setPosASL based on input Z type
ME3D_SETPOS = {
	private ["_side", "_group_id", "_unit_id", "_x", "_x2", "_y", "_y2", "_z", "_z_change", "_z_shift", "_z_type", "_z_array"];
	_side     = _this select 0;
	_group_id = _this select 1;
	_unit_id  = _this select 2;
	_x        = _this select 3;
	_y        = _this select 4;
	_z_change = _this select 5;
	_z_shift  = _this select 6;
	_z_type   = 0;
	_z_array  = [];
	
	// Find Z
	if (ME3D_CAMERA_TARGET == ME3D_TEMPOBJ) then { // preview object
		_unit_id = 2;
		_z_array = ME3D_TEMPOBJ_POS;
		_z_type  = ME3D_TEMPOBJ_Z_TYPE;
	} else {
		if (_unit_id >= 0) then {
			if (_side == "EMPTY") then { // objects
				_z_array = ME3D_VEHICLES_Z;
				_z_type  = ME3D_VEHICLES_Z_TYPE select _unit_id;
			} else {
				if (_side!="MARKER" && _side!="TRIGGER") then { // soldiers
					_z_array = ME3D_GROUPS_Z select _group_id;
					_z_type  = (ME3D_GROUPS_Z_TYPE select _group_id) select _unit_id;
				}
			}
		}
	};
	
	// Move object and update Z array
	if (_z_type > 0) then {
		_z = if (_z_shift) then {(_z_array select _unit_id)+_z_change} else {_z_change};
		_z_array set [_unit_id, _z];
		
		// "setpos:" is the argument for the :class write command so that Fwatch will automatically write "this setpos []" command in the object's initialization field
		if (_z_type == 2) then { //absolute z
			// setPosASL doesn't have the position bug but mission.sqm has
			// so we still need to find offset and compensate :(
			ME3D_CAMERA_TARGET setPos [_x, _y, _z];
			_x2 = 2 * _x - (getPos ME3D_CAMERA_TARGET select 0);
			_y2 = 2 * _y - (getPos ME3D_CAMERA_TARGET select 1);
			
			ME3D_CAMERA_TARGET setPosASL [_x, _y, _z];
			ME3D_THISSETPOS = Format ["""setpos:[%1,(getpos this select 0)+%2,(getpos this select 1)+%3,%4]""", _z_type, (_x2-_x), (_y2-_y), _z];
		} else { //relative z
			ME3D_CAMERA_TARGET setPos [_x, _y, _z];
			_x2 = 2 * _x - (getPos ME3D_CAMERA_TARGET select 0);
			_y2 = 2 * _y - (getPos ME3D_CAMERA_TARGET select 1);
			
			ME3D_CAMERA_TARGET setPos [_x2, _y2, _z];
			ME3D_THISSETPOS = Format ["""setpos:[%1,(getpos this select 0)+%2,(getpos this select 1)+%3,%4]""", _z_type, (_x2-_x)*2, (_y2-_y)*2, _z];
			// multiplying by two because need to compensate twice (for mission.sqm and for using this instance of setpos)
		};
	} else {
		ME3D_CAMERA_TARGET setPos [_x, _y, 0];
		ME3D_CAMERA_TARGET setPos [2 * _x - (getPos ME3D_CAMERA_TARGET select 0), 2 * _y - (getPos ME3D_CAMERA_TARGET select 1), 0];
		ME3D_THISSETPOS = "";
	};
};

// Delete class form the mission.sqm file
ME3D_DELETE_UNIT = {
	private ["_x", "_y", "_dir", "_side", "_group_id", "_unit_id", "_waypoint_id", "_ok", "_classpath"];
	
	_side        = lbData [1, lbCurSel 1];
	_group_id    = call (lbData [2, lbCurSel 2]);
	_group_index = lbValue [2, _group_selection];
	_unit_id     = lbCurSel 3;
	_waypoint_id = lbCurSel 4;
	_classpath   = "";
	_classname   = "";
	_item        = _unit_id;
	_items       = 0;
	_ok          = [];

	// Get classpath and number of items
	if (ME3D_CURRENT_SELECTION_TYPE == "soldier") then {
		if (_side in ["WEST", "EAST", "GUER", "CIV", "LOGIC"]) then {
			_classpath = Format ["Mission,Groups,Item%1,Vehicles", _group_id];
			_classname = Format ["Item%1", _unit_id];
			_items     = count (ME3D_GROUPS_OBJ select _group_index);
		};

		if (_side == "EMPTY") then {
			_classpath = "Mission,Vehicles";
			_classname = Format ["Item%1", _unit_id];
			_items     = count ME3D_VEHICLES_OBJ;
		};

		if (_side == "MARKER") then {
			_classpath = "Mission,Markers";
			_classname = Format ["Item%1", _unit_id];
			_items     = count ME3D_MARKERS_NAME;
		};

		if (_side == "TRIGGER") then {
			_classpath = "Mission,Sensors";
			_classname = Format ["Item%1", _unit_id];
			_items     = count ME3D_SENSORS_TEXT;
		};
	} else {
		_classpath = Format ["[Mission,Groups,Item%1,Waypoints]", _group_id];
		_classname = Format ["Item%1", _waypoint_id];
		_item      = _waypoint_id;
		_items     = count (ME3D_GROUPS_WP select _group_index);
	};

	// Make changes to the sqm
	_ok = call loadFile Format ["\:CLASS WRITE  file:mission.sqm  path:%1  merge:items=%2;deleteclass:%3", _classpath, _items-1, _classname];
	
	if (_ok select 0) then {
		// Rename other classes
		while "_item=_item+1; _item<=_items-1 && (_ok select 0)" do {
			_ok = call loadFile Format ["\:CLASS WRITE  file:mission.sqm  path:%1  renameclass:Item%2  to:Item%3", _classpath, _item, _item-1];
		};
		
		// Make changes to the variables
		if (_ok select 0) then {
			_items = _items - 1;
			if (ME3D_CURRENT_SELECTION_TYPE == "soldier") then {
				if (_side in ["WEST", "EAST", "GUER", "CIV", "LOGIC"]) then {					
					(ME3D_GROUPS_OBJ select _group_index) set [_unit_id, 0];
					ME3D_GROUPS_OBJ set [_group_index, (ME3D_GROUPS_OBJ select _group_index) - [0]];
					
					(ME3D_GROUPS_Z select _group_index) set [_unit_id, objNull];
					ME3D_GROUPS_Z set [_group_index, (ME3D_GROUPS_Z select _group_index) - [objNull]];
					
					(ME3D_GROUPS_Z_TYPE select _group_index) set [_unit_id, objNull];
					ME3D_GROUPS_Z_TYPE set [_group_index, (ME3D_GROUPS_Z_TYPE select _group_index) - [objNull]];
				};

				if (_side == "EMPTY") then {			
					ME3D_VEHICLES_OBJ set [_unit_id, 0];
					ME3D_VEHICLES_OBJ = ME3D_VEHICLES_OBJ - [0];
					
					ME3D_VEHICLES_NAME set [_unit_id, 0];
					ME3D_VEHICLES_NAME = ME3D_VEHICLES_NAME - [0];
					
					ME3D_VEHICLES_Z set [_unit_id, objNull];
					ME3D_VEHICLES_Z = ME3D_VEHICLES_Z - [objNull];
					
					ME3D_VEHICLES_Z_TYPE set [_unit_id, objNull];
					ME3D_VEHICLES_Z_TYPE = ME3D_VEHICLES_Z_TYPE - [objNull];
				};

				if (_side == "MARKER") then {
					ME3D_MARKERS_NAME set [_unit_id, 0];
					ME3D_MARKERS_NAME = ME3D_MARKERS_NAME - [0];
				};

				if (_side == "TRIGGER") then {
					ME3D_SENSORS_TEXT set [_unit_id, 0];
					ME3D_SENSORS_TEXT = ME3D_SENSORS_TEXT - [0];
				};
				
				_unit_selection = _unit_selection - 1;
				if (_unit_selection < 0) then {_unit_selection=0};
			} else {		
				_array = (ME3D_GROUPS_WP select _group_index);
				_array set [_waypoint_id, 0];
				ME3D_GROUPS_WP set [_group_index, _array - [0]];
				
				_waypoint_selection = _waypoint_selection - 1;
				if (_waypoint_selection < 0) then {ME3D_CURRENT_SELECTION_TYPE="soldier"};
			};
			
			if (_items == 0) then {
				// If removed last unit in the group then remove group
				if (ME3D_CURRENT_SELECTION_TYPE == "soldier") then {
					if (_side in ["WEST", "EAST", "GUER", "CIV", "LOGIC"]) then {
						_classpath = "Mission,Groups";
						_classname = Format ["Item%1", _group_id];
						
						_item  = _group_id;
						_items = count ME3D_GROUPS_OBJ;
						
						_ok = call loadFile Format ["\:CLASS WRITE  file:mission.sqm  path:%1  merge:items=%2;deleteclass:%3", _classpath, _items-1, _classname];
						
						if (_ok select 0) then {
							while "_item=_item+1; _item<=_items-1 && (_ok select 0)" do {
								_ok = call loadFile Format ["\:CLASS WRITE  file:mission.sqm  path:%1  renameclass:Item%2  to:Item%3", _classpath, _item, _item-1];
							};
							
							if (_ok select 0) then {
								ME3D_GROUPS_SIDE set [_group_index, 0];
								ME3D_GROUPS_SIDE = ME3D_GROUPS_SIDE - [0];
								
								ME3D_GROUPS_OBJ set [_group_index, 0];
								ME3D_GROUPS_OBJ = ME3D_GROUPS_OBJ - [0];
								
								ME3D_GROUPS_WP set [_group_index, 0];
								ME3D_GROUPS_WP = ME3D_GROUPS_WP - [0];
								
								ME3D_GROUPS_Z set [_group_index, objNull];
								ME3D_GROUPS_Z = ME3D_GROUPS_Z - [objNull];
								
								ME3D_GROUPS_Z_TYPE set [_group_index, objNull];
								ME3D_GROUPS_Z_TYPE = ME3D_GROUPS_Z_TYPE - [objNull];
								
								_group_selection = _group_selection - 1;
								if (_group_selection < 0) then {
									_group_selection = 0
								};
								
								if ("_x==_side" count ME3D_GROUPS_SIDE == 0) then {
									_side_selection = _side_selection - 1;
									if (_side_selection < 0) then {_side_selection=0};			
								};
							}
						}
					};
				} else {
					ME3D_CURRENT_SELECTION_TYPE = "soldier"
				};
			};
		}
	};
	
	if (!(_ok select 0)) then {
		//player globalchat format ["%1 %2", _classpath, _classname]; player sidechat format ["%1 %2", _item, _items];
		player groupChat (_ok select 3);
	}
};

// Create a new variable name (for the new unit)
ME3D_GENERATE_VARIABLE_NAME = {
	private ["_prefix", "_return", "_counter"];
	
	_prefix = _this;
	_return = _prefix;
	
	if (Format ["%1", call _prefix] != "scalar bool array string 0xfcffffef") then {
		_counter = 0;
		
		while {_counter=_counter+1; _return=Format ["%1_%2",_prefix,_counter]; Format ["%1",call _return]!="scalar bool array string 0xfcffffef"} do {};
	};

	_return
};

// Handle mouse double-click
ME3D_MOUSE_DOUBLE_CLICK = {
	_cursor_x = (FWATCH_INPUT_MULTI select 3) select 2;
	_cursor_y = (FWATCH_INPUT_MULTI select 3) select 3;
	_over_soldier_list_box  = _cursor_x > ((_ui select 2) select 0) && _cursor_x < ((_ui select 2) select 1)  &&  _cursor_y > ((_ui select 2) select 2) && _cursor_y < ((_ui select 2) select 3);
	_over_waypoint_list_box = _cursor_x > ((_ui select 3) select 0) && _cursor_x < ((_ui select 3) select 1)  &&  _cursor_y > ((_ui select 3) select 2) && _cursor_y < ((_ui select 3) select 3);
	_over_objlist_list_box  = _cursor_x > ((_ui select 1) select 0) && _cursor_x < ((_ui select 1) select 1)  &&  _cursor_y > ((_ui select 2) select 2) && _cursor_y < ((_ui select 2) select 3);

	// unit selection
	if (_over_soldier_list_box && (ME3D_CURRENT_SELECTION_TYPE != "soldier" || ME3D_CAMERA_TARGET == ME3D_TEMPOBJ)) then {"soldier" call ME3D_REFRESH_UNIT_SELECTION};
	if (_over_waypoint_list_box && (ME3D_CURRENT_SELECTION_TYPE != "waypoint" || ME3D_CAMERA_TARGET == ME3D_TEMPOBJ)) then {"waypoint" call ME3D_REFRESH_UNIT_SELECTION};
	if (_over_objlist_list_box && ME3D_CAMERA_TARGET != ME3D_TEMPOBJ && lbCurSel 8 >= 0) then {[lbData [8, lbCurSel 8], lbValue [8, lbCurSel 8], false] call ME3D_REFRESHME3D_TEMPOBJ};
	
	
	// double-click
	private ["_time1", "_time2"];
	
	if (count _double_click < 2) then {
		_double_click set [count _double_click, _time];
	}; 
	
	if (count _double_click>=2) then {
		_time1       = _double_click select 0;
		_time2       = _double_click select 1;
		_double_click = [];
		
		if (_time2-_time1 < 0.25 && ME3D_CURSOR_OVER_BACKGROUND) then {
			if (_over_soldier_list_box) then {
				goto "EditUnit"
			} else {
				if (_over_waypoint_list_box) then {
					ME3D_CURRENT_SELECTION_TYPE = "waypoint";
					_group_id                   = call (lbData [2, lbCurSel 2]);
					
					if (count (ME3D_GROUPS_WP select _group_id) == 0) then {
						goto "InsertUnit"
					} else {
						goto "EditUnit"
					}
				} else {
					if (_over_objlist_list_box) then {
						ME3D_CURRENT_SELECTION_TYPE="soldier";
						goto "InsertUnit"
					}
				}
			}			
		}
	}
};








//---- Format templates used when adding a new class to the sqm ------------------------

// Class entry added to mission.sqm when you add a new unit
ME3D_TEMPLATE_NEW_UNIT = {				items=%1; 
				class Item%2 {
					position[]=%3; 
					azimut=%4; 
					id=%5; 
					side="%6"; 
					vehicle="%7"; 
					leader=%8; 
					skill=%9; 
					text="%10";
				};
};

// Class entry added to mission.sqm when you add a new group
ME3D_TEMPLATE_NEW_GROUP = {		items=%1;
		class Item%2 
		{
			side="%3";
			class Vehicles {
				items=0;
			};
		};
};

// Class entry merged with mission.sqm when you add a new waypoint
ME3D_TEMPLATE_NEW_WAYPOINT = {			class Waypoints
			{
				items=%1; 
				class Item%2 
				{
					position[]=%3;
					description="Waypoint #%4";
					class Effects {};
					showWP="NEVER";
				};
			};
};

// Class entry added to mission.sqm when you add a new marker
ME3D_TEMPLATE_NEW_MARKER = {			class Markers
	{
		items=%1;
		class Item%2 
		{
			position[]=%3; 
			name="%4"; 
			type="%5";
			colorName="%6";
			a=%7;
			b=%8;
		};
	};
};

// Class entry added to mission.sqm when you add a new trigger
ME3D_TEMPLATE_NEW_TRIGGER = {			class Sensors
	{
		items=%1;
		class Item%2 
		{
			position[]=%3; 
			age="UNKNOWN"; 
			text="%4"; 
			class Effects {};
		};
	};
};

// Class entry merged with mission.sqm when you add a new empty vehicle
ME3D_TEMPLATE_NEW_VEHICLE = {			class Vehicles
			{
				items=%1; 
				class Item%2 
				{
					position[]=%3; 
					id=%4; 
					side="%5"; 
					vehicle="%6"; 
					skill=0.6; 
					text="%7";
				};
			};
};
